@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require "stringio"
 
 module YARD
@@ -119,11 +120,9 @@ module YARD
       #
       # @return [String] POT format string
       def generate
-        pot = header
+        pot = String.new(header)
         sorted_messages = @messages.sort_by do |message|
-          sorted_locations = message.locations.sort_by do |location|
-            location
-          end
+          sorted_locations = message.locations.sort
           sorted_locations.first || []
         end
         sorted_messages.each do |message|
@@ -133,6 +132,7 @@ module YARD
       end
 
       private
+
       def header
         <<-EOH
 # SOME DESCRIPTIVE TITLE.
@@ -192,7 +192,7 @@ EOH
       end
 
       def extract_documents(object)
-        return if @extracted_objects.has_key?(object)
+        return if @extracted_objects.key?(object)
 
         @extracted_objects[object] = true
         case object
@@ -235,17 +235,19 @@ EOH
       def extract_tag_documents(tag)
         extract_tag_name(tag)
         extract_tag_text(tag)
+
+        extract_documents(tag) if Tags::OverloadTag === tag
       end
 
       def extract_tag_name(tag)
         return if tag.name.nil?
-        return if tag.name.is_a?(String) and tag.name.empty?
+        return if tag.name.is_a?(String) && tag.name.empty?
         key = "tag|#{tag.tag_name}|#{tag.name}"
         message = register_message(key)
         tag.object.files.each do |path, line|
           message.add_location(path, line)
         end
-        tag_label = "@#{tag.tag_name}"
+        tag_label = String.new("@#{tag.tag_name}")
         tag_label << " [#{tag.types.join(', ')}]" if tag.types
         message.add_comment(tag_label)
       end
@@ -257,7 +259,7 @@ EOH
         tag.object.files.each do |path, line|
           message.add_location(path, line)
         end
-        tag_label = "@#{tag.tag_name}"
+        tag_label = String.new("@#{tag.tag_name}")
         tag_label << " [#{tag.types.join(', ')}]" if tag.types
         tag_label << " #{tag.name}" if tag.name
         message.add_comment(tag_label)
