@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module YARD
   module Parser
     module Ruby
@@ -92,13 +93,13 @@ module YARD
 
         # List of all known keywords
         # @return [Hash]
-        KEYWORDS = { :class => true, :alias => true, :lambda => true, :do_block => true,
+        KEYWORDS = {:class => true, :alias => true, :lambda => true, :do_block => true,
           :def => true, :defs => true, :begin => true, :rescue => true, :rescue_mod => true,
           :if => true, :if_mod => true, :else => true, :elsif => true, :case => true,
           :when => true, :next => true, :break => true, :retry => true, :redo => true,
           :return => true, :throw => true, :catch => true, :until => true, :until_mod => true,
           :while => true, :while_mod => true, :yield => true, :yield0 => true, :zsuper => true,
-          :unless => true, :unless_mod => true, :for => true, :super => true, :return0 => true }
+          :unless => true, :unless_mod => true, :for => true, :super => true, :return0 => true}
 
         # @group Creating an AstNode
 
@@ -128,7 +129,7 @@ module YARD
               ReferenceNode
             elsif type.to_s =~ /_literal\Z/
               LiteralNode
-            elsif KEYWORDS.has_key?(type)
+            elsif KEYWORDS.key?(type)
               KeywordNode
             else
               AstNode
@@ -157,13 +158,14 @@ module YARD
           @fallback_line = opts[:listline]
           @fallback_source = opts[:listchar]
           @token = true if opts[:token]
+          @docstring = nil
         end
 
         # @return [Boolean] whether the node is equal to another by checking
         #   the list and type
         # @private
-        def ==(ast)
-          super && type == ast.type
+        def ==(other)
+          super && type == other.type
         end
 
         # @group Traversing a Node
@@ -207,7 +209,7 @@ module YARD
           nodes = [self]
           nodes.each.with_index do |node, index|
             yield node
-            nodes.insert index+1, *node.children
+            nodes.insert index + 1, *node.children
           end
         end
 
@@ -285,18 +287,16 @@ module YARD
 
         # @return [nil] pretty prints the node
         def pretty_print(q)
-          objs = self.dup + [:__last__]
+          objs = dup + [:__last__]
           objs.unshift(type) if type && type != :list
 
           options = []
-          if @docstring
-            options << ['docstring', docstring]
-          end
+          options << ['docstring', docstring] if @docstring
           if @source_range || @line_range
             options << ['line', line_range]
             options << ['source', source_range]
           end
-          objs.pop if options.size == 0
+          objs.pop if options.empty?
 
           q.group(3, 's(', ')') do
             q.seplist(objs, nil, :each) do |v|
@@ -341,8 +341,9 @@ module YARD
           if size == 0
             self.line_range = @fallback_line
             self.source_range = @fallback_source
-          elsif children.size > 0
-            f, l = children.first, children.last
+          elsif !children.empty?
+            f = children.first
+            l = children.last
             self.line_range = Range.new(f.line_range.first, l.line_range.last)
             self.source_range = Range.new(f.source_range.first, l.source_range.last)
           elsif @fallback_line || @fallback_source
@@ -376,18 +377,47 @@ module YARD
       end
 
       class ParameterNode < AstNode
-        def required_params; self[0] end
-        def required_end_params; self[3] end
-        def splat_param; self[2] ? self[2][0] : nil end
-        def block_param; self[-1] ? self[-1][0] : nil end
-        def optional_params
-          optional = self[1] || []
-          if self[-3] && self[-3][0] && self[-3][0].type == :default_arg
-            optional += self[-3]
+        def unnamed_required_params
+          self[0]
+        end
+
+        def unnamed_optional_params
+          return @unnamed_optional_params if defined?(@unnamed_optional_params)
+
+          params = self[1] || []
+          if self[-3] && self[-3][0] && self[-3][0].type == :unnamed_optional_arg
+            params += self[-3]
           end
-          optional.empty? ? nil : optional
+
+          @unnamed_optional_params = params.empty? ? nil : params
+        end
+
+        def named_params
+          return @named_params if defined?(@named_params)
+
+          if YARD.ruby2? && self[-3] && self[-3][0] && self[-3][0].type == :named_arg
+            @named_params = self[-3]
+          else
+            @named_params = nil
+          end
+        end
+
+        def splat_param
+          self[2] ? self[2][0] : nil
+        end
+
+        def unnamed_end_params
+          self[3]
+        end
+
+        def double_splat_param
+          return nil unless YARD.ruby2?
+          self[-2] if self[-2].is_a?(AstNode) && self[-2].type == :ident
+        end
+
+        def block_param
+          self[-1] ? self[-1][0] : nil
         end
-        def keyword_param; YARD.ruby2? ? self[-2] : nil end
       end
 
       class MethodCallNode < AstNode
@@ -396,7 +426,13 @@ module YARD
 
         def method_name(name_only = false)
           name = self[index_adjust]
-          name_only ? name.jump(:ident).first.to_sym : name
+          if name == :call
+            nil
+          elsif name_only && Array === name
+            name.jump(:ident).first.to_sym
+          else
+            name
+          end
         end
 
         def parameters(include_block_param = true)
@@ -409,6 +445,7 @@ module YARD
         end
 
         def block_param; parameters.last end
+
         def block
           last.type == :do_block || last.type == :brace_block ? last : nil
         end
@@ -455,9 +492,8 @@ module YARD
         def then_block; self[1] end
 
         def else_block
-          if self[2] && !cmod?
-            self[2].type == :elsif ? self[2] : self[2][0]
-          end
+          return unless self[2] && !cmod?
+          self[2].type == :elsif ? self[2] : self[2][0]
         end
 
         private
@@ -493,4 +529,4 @@ module YARD
       end
     end
   end
-end
\ No newline at end of file
+end
