@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'cgi'
 
 module YARD
@@ -7,6 +8,9 @@ module YARD
       include MarkupHelper
       include HtmlSyntaxHighlightHelper
 
+      # @private
+      URLMATCH = /[^\w\s~!\*'\(\):;@&=\$,\[\]<>-]/
+
       # @group Escaping Template Data
 
       # Escapes HTML entities
@@ -22,9 +26,23 @@ module YARD
       # @param [String] text the URL
       # @return [String] the escaped URL
       def urlencode(text)
-        CGI.escape(text.to_s)
+        text = text.dup
+        enc = nil
+        if text.respond_to?(:force_encoding)
+          enc = text.encoding
+          text = text.force_encoding('binary')
+        end
+
+        text = text.gsub(/%[a-z0-9]{2}|#{URLMATCH}/i) do
+          $&.size > 1 ? $& : "%" + $&.ord.to_s(16).upcase
+        end.tr(' ', '+')
+
+        text = text.force_encoding(enc) if enc
+        text
       end
 
+      module_function :urlencode
+
       # @group Converting Markup to HTML
 
       # Turns text into HTML using +markup+ style formatting.
@@ -38,7 +56,7 @@ module YARD
         return text unless respond_to?(markup_meth)
         return "" unless text
         return text unless markup
-        html = send(markup_meth, text)
+        html = send(markup_meth, text).dup
         if html.respond_to?(:encode)
           html = html.force_encoding(text.encoding) # for libs that mess with encoding
           html = html.encode(:invalid => :replace, :replace => '?')
@@ -61,7 +79,8 @@ module YARD
           provider.new(text, :autolink).to_html
         elsif provider.to_s == 'RedcarpetCompat'
           provider.new(text, :no_intraemphasis, :gh_blockcode,
-                             :fenced_code, :autolink).to_html
+                             :fenced_code, :autolink, :tables,
+                             :lax_spacing).to_html
         else
           provider.new(text).to_html
         end
@@ -187,8 +206,13 @@ module YARD
       # @return [String] HTML with linkified references
       def resolve_links(text)
         code_tags = 0
-        text.gsub(/<(\/)?(pre|code|tt)|(\\|!)?\{(?!\})(\S+?)(?:\s([^\}]*?\S))?\}(?=[\W<]|.+<\/|$)/m) do |str|
-          closed, tag, escape, name, title, match = $1, $2, $3, $4, $5, $&
+        text.gsub(%r{<(/)?(pre|code|tt)|(\\|!)?\{(?!\})(\S+?)(?:\s([^\}]*?\S))?\}(?=[\W<]|.+</|$)}m) do |str|
+          closed = $1
+          tag = $2
+          escape = $3
+          name = $4
+          title = $5
+          match = $&
           if tag
             code_tags += (closed ? -1 : 1)
             next str
@@ -197,10 +221,11 @@ module YARD
 
           next(match[1..-1]) if escape
 
-          next(match) if name[0,1] == '|'
+          next(match) if name[0, 1] == '|'
 
-          if name == '<a' && title =~ /href=["'](.+?)["'].*>.*<\/a>\s*(.*)\Z/
-            name, title = $1, $2
+          if name == '<a' && title =~ %r{href=["'](.+?)["'].*>.*</a>\s*(.*)\Z}
+            name = $1
+            title = $2
             title = nil if title.empty?
           end
 
@@ -210,12 +235,12 @@ module YARD
             object
           else
             link = linkify(name, title)
-            if (link == name || link == title) && (name+' '+link !~ /\A<a\s.*>/)
+            if (link == name || link == title) && (name + ' ' + link !~ /\A<a\s.*>/)
               match = /(.+)?(\{#{Regexp.quote name}(?:\s.*?)?\})(.+)?/.match(text)
-              file = (@file ? @file.filename : object.file) || '(unknown)'
-              line = (@file ? 1 : (object.docstring.line_range ? object.docstring.line_range.first : 1)) + (match ? $`.count("\n") : 0)
-              log.warn "In file `#{file}':#{line}: Cannot resolve link to #{name} from text" + (match ? ":" : ".")
-              log.warn((match[1] ? '...' : '') + match[2].gsub("\n","") + (match[3] ? '...' : '')) if match
+              file = (defined?(@file) && @file ? @file.filename : object.file) || '(unknown)'
+              line = (defined?(@file) && @file ? 1 : (object.docstring.line_range ? object.docstring.line_range.first : 1)) + (match ? $`.count("\n") : 0)
+              log.warn "In file `#{file}':#{line}: Cannot resolve link to #{name} from text" + (match ? ":" : ".") +
+                       "\n\t" + (match[1] ? '...' : '') + match[2].delete("\n") + (match[3] ? '...' : '') if match
             end
 
             link
@@ -241,12 +266,17 @@ module YARD
           file = CodeObjects::ExtraFileObject.new(file)
         end
         file.attributes[:markup] ||= markup_for_file('', file.filename)
-        htmlify(file.contents, file.attributes[:markup] || options.markup)
+        insert_include(file.contents, file.attributes[:markup] || options.markup)
       end
 
       # (see BaseHelper#link_include_object)
       def link_include_object(obj)
-        htmlify(obj.docstring)
+        insert_include(obj.docstring)
+      end
+
+      # Inserts an include link while respecting inlining
+      def insert_include(text, markup = options.markup)
+        htmlify(text, markup).gsub(%r{\A\s*<p>|</p>\s*\Z}, '')
       end
 
       # (see BaseHelper#link_object)
@@ -267,7 +297,7 @@ module YARD
             title = h(object.relative_path(obj))
           end
         else
-          title = h(obj.to_s)
+          title = h(obj.title)
         end
         return title unless serializer
         return title if obj.is_a?(CodeObjects::Proxy)
@@ -280,12 +310,12 @@ module YARD
       # (see BaseHelper#link_url)
       def link_url(url, title = nil, params = {})
         title ||= url
-        title.gsub!(/[\r\n]/, ' ')
+        title = title.gsub(/[\r\n]/, ' ')
         params = SymbolHash.new(false).update(
           :href => url,
-          :title  => h(title)
+          :title => h(title)
         ).update(params)
-        params[:target] ||= '_parent' if url =~ /^(\w+):\/\//
+        params[:target] ||= '_parent' if url =~ %r{^(\w+)://}
         "<a #{tag_attrs(params)}>#{title}</a>".gsub(/[\r\n]/, ' ')
       end
 
@@ -321,16 +351,18 @@ module YARD
 
         if obj.is_a?(CodeObjects::Base) && !obj.is_a?(CodeObjects::NamespaceObject)
           # If the obj is not a namespace obj make it the anchor.
-          anchor, obj = obj, obj.namespace
+          anchor = obj
+          obj = obj.namespace
         end
 
         objpath = serializer.serialized_path(obj)
         return link unless objpath
 
+        relative = false if object == Registry.root
         if relative
           fromobj = object
           if object.is_a?(CodeObjects::Base) &&
-              !object.is_a?(CodeObjects::NamespaceObject)
+             !object.is_a?(CodeObjects::NamespaceObject)
             fromobj = owner
           end
 
@@ -343,6 +375,9 @@ module YARD
         link + (anchor ? '#' + urlencode(anchor_for(anchor)) : '')
       end
 
+      alias mtime_url url_for
+      def mtime(_file) nil end
+
       # Returns the URL for a specific file
       #
       # @param [String, CodeObjects::ExtraFileObject] filename the filename to link to
@@ -355,11 +390,8 @@ module YARD
           fromobj = fromobj.namespace
         end
         from = serializer.serialized_path(fromobj)
-        if filename == options.readme
-          path = 'index.html'
-        else
-          path = serializer.serialized_path(filename)
-        end
+        path = filename == options.readme ?
+          'index.html' : serializer.serialized_path(filename)
         link = File.relative_path(from, path)
         link += (anchor ? '#' + urlencode(anchor) : '')
         link
@@ -460,7 +492,7 @@ module YARD
         elsif !type.empty?
           type = link ? h(type) : format_types([type], false)
         end
-        type = "(#{type}) " unless type.empty?
+        type = "#{type} " unless type.empty?
         type
       end
 
@@ -477,14 +509,16 @@ module YARD
         meth = convert_method_to_overload(meth)
 
         type = signature_types(meth, link)
-        scope = meth.scope == :class ? "+" : "-"
+        type = "&#x21d2; #{type}" if type && !type.empty?
+        scope = meth.scope == :class ? "." : "#"
         name = full_attr_name ? meth.name : meth.name.to_s.gsub(/^(\w+)=$/, '\1')
         blk = format_block(meth)
         args = !full_attr_name && meth.writer? ? "" : format_args(meth)
         extras = []
         extras_text = ''
         if show_extras
-          if rw = meth.attr_info
+          rw = meth.attr_info
+          if rw
             attname = [rw[:read] ? 'read' : nil, rw[:write] ? 'write' : nil].compact
             attname = attname.size == 1 ? attname.join('') + 'only' : nil
             extras << attname if attname
@@ -492,7 +526,7 @@ module YARD
           extras << meth.visibility if meth.visibility != :public
           extras_text = ' <span class="extras">(' + extras.join(", ") + ')</span>' unless extras.empty?
         end
-        title = "%s %s<strong>%s</strong>%s %s" % [scope, type, h(name), args, blk]
+        title = "%s<strong>%s</strong>%s %s %s" % [scope, h(name), args, blk, type]
         if link
           if meth.is_a?(YARD::CodeObjects::MethodObject)
             link_title = "#{h meth.name(true)} (#{meth.scope} #{meth.type})"
@@ -518,16 +552,18 @@ module YARD
       # @since 0.5.4
       def charset
         has_encoding = defined?(::Encoding)
-        if @file && has_encoding
+        if defined?(@file) && @file && has_encoding
           lang = @file.contents.encoding.to_s
         else
-          return 'utf-8' unless has_encoding || lang = ENV['LANG']
-          if has_encoding
-            lang = ::Encoding.default_external.name.downcase
-          else
-            lang = lang.downcase.split('.').last
-          end
+          return 'utf-8' unless has_encoding || ENV['LANG']
+          lang =
+            if has_encoding
+              ::Encoding.default_external.name.downcase
+            else
+              ENV['LANG'].downcase.split('.').last
+            end
         end
+
         case lang
         when "ascii-8bit", "us-ascii", "ascii-7bit"; 'iso-8859-1'
         when "utf8"; 'utf-8'
@@ -544,7 +580,7 @@ module YARD
       # @param [Hash{String => String}] opts the tag options
       # @return [String] the tag attributes of an HTML tag
       def tag_attrs(opts = {})
-        opts.sort_by {|k, v| k.to_s }.map {|k,v| "#{k}=#{v.to_s.inspect}" if v }.join(" ")
+        opts.sort_by {|k, _v| k.to_s }.map {|k, v| "#{k}=#{v.to_s.inspect}" if v }.join(" ")
       end
 
       # Converts a {CodeObjects::MethodObject} into an overload object
@@ -567,7 +603,8 @@ module YARD
       def parse_lang_for_codeblock(source)
         type = nil
         if source =~ /\A(?:[ \t]*\r?\n)?[ \t]*!!!([\w.+-]+)[ \t]*\r?\n/
-          type, source = $1, $'
+          type = $1
+          source = $'
         end
 
         [type, source]
@@ -580,20 +617,19 @@ module YARD
       # @return [String] highlighted html
       # @see #html_syntax_highlight
       def parse_codeblocks(html)
-        html.gsub(/<pre\s*(?:lang="(.+?)")?>(?:\s*<code\s*(?:class="(.+?)")?\s*>)?(.+?)(?:<\/code>\s*)?<\/pre>/m) do
+        html.gsub(%r{<pre\s*(?:lang="(.+?)")?>(?:\s*<code\s*(?:class="(.+?)")?\s*>)?(.+?)(?:</code>\s*)?</pre>}m) do
           string = $3
           # handle !!!LANG prefix to send to html_syntax_highlight_LANG
-          language, _ = parse_lang_for_codeblock(string)
+          language, = parse_lang_for_codeblock(string)
           language ||= $1 || $2 || object.source_type
 
           if options.highlight
             string = html_syntax_highlight(CGI.unescapeHTML(string), language)
           end
           classes = ['code', language].compact.join(' ')
-          %Q{<pre class="#{classes}"><code class="#{language}">#{string}</code></pre>}
+          %(<pre class="#{classes}"><code class="#{language}">#{string}</code></pre>)
         end
       end
     end
   end
 end
-
